require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfscontactperson',
  dscmeta_resource_friendly_name: 'AdfsContactPerson',
  dscmeta_resource_name: 'MSFT_AdfsContactPerson',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsContactPerson resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_federationservicename: {
      type: 'String',
      desc: 'Specifies the name of the Federation Service.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_emailaddress: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of e-mail addresses of the contact person.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_surname: {
      type: 'Optional[String]',
      desc: 'Specifies the surname, or last name, of the contact person.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_telephonenumber: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of telephone numbers of the contact person.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_givenname: {
      type: 'Optional[String]',
      desc: 'Specifies the given name of the contact person.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_company: {
      type: 'Optional[String]',
      desc: 'Specifies the company name of the contact person.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
