require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfsfarm',
  dscmeta_resource_friendly_name: 'AdfsFarm',
  dscmeta_resource_name: 'MSFT_AdfsFarm',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsFarm resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_federationservicedisplayname: {
      type: 'Optional[String]',
      desc: 'Specifies the display name of the Federation Service.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlconnectionstring: {
      type: 'Optional[String]',
      desc: 'Specifies the SQL Server database that will store the AD FS configuration settings. If not specified, the AD FS installer uses the Windows Internal Database to store configuration settings.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_certificatethumbprint: {
      type: 'String',
      desc: 'Specifies the thumbprint of the certificate to use for HTTPS bindings and service communication for ADFS. This value should match the thumbprint of a valid certificate in the Local Computer certificate store.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: 'Optional[String]',
      desc: 'The state of the ADFS Farm.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_adminconfiguration: {
      type: 'Optional[Array[Struct[{
              key => Optional[String],
              value => Optional[String],
            }]]]',
      desc: 'Specifies the Admin Configuration',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_KeyValuePair[]',
      mof_is_embedded: true,
    },
    dsc_federationservicename: {
      type: 'String',
      desc: 'Specifies the DNS name of the federation service. This value must match the subject name of the specified certificate.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_overwriteconfiguration: {
      type: 'Optional[Boolean]',
      desc: 'This parameter must be used to remove an existing Active Directory Federation Services (AD FS) configuration database and overwrite it with a new database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_serviceaccountcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the Active Directory account under which the AD FS service runs in the form: <domain name>\<user name>.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies a PSCredential object that must have domain administrator privileges.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_groupserviceaccountidentifier: {
      type: 'Optional[String]',
      desc: 'Specifies the Group Managed Service Account under which the Active Directory Federation Services (AD FS) service runs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
