require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfsfarmnode',
  dscmeta_resource_friendly_name: 'AdfsFarmNode',
  dscmeta_resource_name: 'MSFT_AdfsFarmNode',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsFarmNode resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_federationservicename: {
      type: 'String',
      desc: 'Specifies the DNS name of the federation service.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlconnectionstring: {
      type: 'Optional[String]',
      desc: 'Specifies the SQL Server database that will store the AD FS configuration settings. If not specified, AD FS uses Windows Internal Database to store configuration settings.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_certificatethumbprint: {
      type: 'String',
      desc: 'Specifies the value of the certificate thumbprint of the certificate that should be used in the SSL binding of the Default Web Site in IIS. This value should match the thumbprint of a valid certificate in the Local Computer certificate store.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: 'Optional[String]',
      desc: 'The state of the ADFS Farm.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_groupserviceaccountidentifier: {
      type: 'Optional[String]',
      desc: 'Specifies the Group Managed Service Account under which the Active Directory Federation Services (AD FS) service runs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_overwriteconfiguration: {
      type: 'Optional[Boolean]',
      desc: 'This parameter must be used to remove an existing AD FS configuration database and overwrite it with a new database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_serviceaccountcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the Active Directory account under which the AD FS service runs. All nodes in the farm must use the same service account.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies a PSCredential object that must have domain administrator privileges.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_primarycomputerport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the primary computer port. The computer uses the HTTP port that you specify to connect with the primary computer in order to synchronize configuration settings. Specify a value of 80 for this parameter, or specify an alternate value if the HTTP port on the primary computer is not 80. If this parameter is not specified, a default port value of 80 is assumed.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_primarycomputername: {
      type: 'Optional[String]',
      desc: 'Specifies the name of the primary in a farm. The cmdlet adds the computer to the farm that has the primary that you specify.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
