require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfsglobalauthenticationpolicy',
  dscmeta_resource_friendly_name: 'AdfsGlobalAuthenticationPolicy',
  dscmeta_resource_name: 'MSFT_AdfsGlobalAuthenticationPolicy',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsGlobalAuthenticationPolicy resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_federationservicename: {
      type: 'String',
      desc: 'Specifies the DNS name of the federation service.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_clientauthenticationmethods: {
      type: "Optional[Array[Enum['ClientSecretPostAuthentication', 'ClientSecretBasicAuthentication', 'PrivateKeyJWTBearerAuthentication', 'WindowsIntegratedAuthentication', 'None']]]",
      desc: 'Specifying this parameter configures an external authentication provider, for second stage authentication, in the global policy',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_primaryintranetauthenticationprovider: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of names of authentication providers for the primary intranet to add to the global policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_primaryextranetauthenticationprovider: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of names of authentication providers for the primary extranet to add to the global policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_deviceauthenticationmethod: {
      type: "Optional[Enum['All', 'ClientTLS', 'SignedToken']]",
      desc: 'Specifying this parameter configures an external authentication provider, for second stage authentication, in the global policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowadditionalauthenticationasprimary: {
      type: 'Optional[Boolean]',
      desc: 'Specifying this parameter configures an external authentication provider for second stage authentication in the global policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_additionalauthenticationprovider: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of names of external authentication providers to add to the global policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_windowsintegratedfallbackenabled: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether fallback to Integrated Windows Authentication is enabled on the intranet.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_deviceauthenticationenabled: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether device authentication is enabled for the global policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_enablepaginatedauthenticationpages: {
      type: 'Optional[Boolean]',
      desc: 'Enable the paginated authentication sign-in experience. This is only supported on Windows Server 2019 and above.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
