require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfsglobalwebcontent',
  dscmeta_resource_friendly_name: 'AdfsGlobalWebContent',
  dscmeta_resource_name: 'MSFT_AdfsGlobalWebContent',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsGlobalWebContent resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_homerealmdiscoveryotherorganizationdescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the text for the home realm discovery description for other organization.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_errorpagedeviceauthenticationerrormessage: {
      type: 'Optional[String]',
      desc: 'Specifies an error message to display for any device authentication errors that occur for a token request. Device authentication errors occur when the user presents an expired user@device certificate to AD FS, a certificate is not found in AD DS, or a certificate is disabled in AD DS. This string can be an HTML fragment.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_helpdesklinktext: {
      type: 'Optional[String]',
      desc: 'Specifies the help desk link text that is shown on the logon pages for AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_errorpagegenericerrormessage: {
      type: 'Optional[String]',
      desc: 'Specifies an error message to display for any generic errors that occur for a token request. This string can be an HTML fragment.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_privacylink: {
      type: 'Optional[String]',
      desc: 'Specifies the Privacy policy link that is shown on the logon pages for AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_companyname: {
      type: 'Optional[String]',
      desc: 'Specifies the company name. AD FS displays the company name in the sign-in pages when you have not set a logo on the active web theme.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_errorpageauthorizationerrormessage: {
      type: 'Optional[String]',
      desc: 'Specifies an error message to display when a user encounters any authorization errors that occur for a token request. This string can be an HTML fragment.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_updatepasswordpagedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the description to display in the update password page when users change their passwords.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_locale: {
      type: 'String',
      desc: 'Specifies a locale. The cmdlet sets global web content for the locale that you specify.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_homerealmdiscoverypagedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the text for the home realm discovery page description.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_errorpagedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies an error message to display when a user encounters any generic errors that occur for a token request. This string can be an HTML fragment.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_homelinktext: {
      type: 'Optional[String]',
      desc: 'Specifies the Home link text that is shown on the logon pages for AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_signinpagedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the description to display when a user signs in to applications by using AD FS. When you use Integrated Windows Authentication in the intranet, users do not see this page.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_homelink: {
      type: 'Optional[String]',
      desc: 'Specifies the Home link that is shown on the logon pages for AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_federationservicename: {
      type: 'String',
      desc: 'Specifies the DNS name of the federation service.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_privacylinktext: {
      type: 'Optional[String]',
      desc: 'Specifies the Privacy policy link text that is shown on the logon pages for AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_helpdesklink: {
      type: 'Optional[String]',
      desc: 'Specifies the help desk link that is shown on the logon pages for AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_certificatepagedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the text on the certificate page. Active Directory Federation Services (AD FS) displays the text that you specify when it prompts the user for a certificate.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_errorpagesupportemail: {
      type: 'Optional[String]',
      desc: 'Specifies the support email address on the error page.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_signoutpagedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the description to display when a user signs out of applications.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_organizationalnamedescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies text for the organizational name description.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_signinpageadditionalauthenticationdescriptiontext: {
      type: 'Optional[String]',
      desc: 'Specifies the description to display when an application prompts a user for additional authentication. The sign-in page can also display a description that is provided by the additional authentication provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
