require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfsproperties',
  dscmeta_resource_friendly_name: 'AdfsProperties',
  dscmeta_resource_name: 'MSFT_AdfsProperties',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsProperties resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_signsamlauthnrequests: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the Federation Service signs SAML protocol authentication requests to claims providers.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certificategenerationthreshold: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the period of time, in days, that any certificates that the Federation Service generates remain valid. The default value is 365 days.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_enablekmsi: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether to enable the Keep Me Signed In (KMSI) option for form-based authentication. KMSI is limited to providing only 24 hours of SSO. Note that a workplace joined device gets 7 days of SSO by default and does not need this option enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_enableloopdetection: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether to enable loop detection. Loops occur when a relying party continuously rejects a valid security token and redirects back to AD FS. The cycle terminates after 6 loops have been detected.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_extranetlockoutmode: {
      type: "Optional[Enum['ADFSSmartLockoutLogOnly', 'ADFSSmartLockoutEnforce']]",
      desc: 'Specifies the maximum number of bad password attempts permitted against the directory before the account is throttled when accessing applications from the extranet for familiar locations. If you use Active Directory Domain Services account lockout policies, it is strongly recommended that you set this threshold to a value that is less than the threshold in AD DS to avoid lockout of the user inside and outside the network. This property is only supported in Windows Server 2019 and above.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_extranetobservationwindow: {
      type: 'Optional[String]',
      desc: 'Specifies the timespan of the lockout observation window. AD FS will reset a throttled state of an account when more than one observation window has expired since the last bad password attempt, as reported by Active Directory Domain Services. It is also possible that the last bad password field in AD DS is cleared by AD DS based on its own observation windows. In this case, AD FS will allow the request to be passed onto AD DS for validation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_globalrelyingpartyclaimsissuancepolicy: {
      type: 'Optional[String]',
      desc: 'Specifies a global relying party claims issuance policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_certificatecriticalthreshold: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the period of time, in days, prior to the expiration of a current primary signing or decryption certificate. When a certificate reaches this threshold, the Federation Service initiates the automatic certificate rollover service, generates a new certificate, and promotes it as the primary certificate. This rollover process occurs even if the critical threshold interval does not provide sufficient time for partners to replicate the new metadata. Specify a short period of time that is used only in extreme conditions when the Federation Service has not been able to generate a new certificate in advance.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_monitoringinterval: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the frequency, in minutes, with which the Federation Service monitors the federation metadata of relying parties and claims providers that are enabled for federation metadata monitoring.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_tlsclientport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the port number where AD FS listens for user certificate authentication requests. Use this only when user certificate authentication is used in AD FS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_proxytrusttokenlifetime: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the valid token lifetime, in minutes, for proxy trust tokens. This value is used by the federation server proxy to authenticate with its associated federation server.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_wiasupporteduseragents: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of acceptable user agents that support seamless sign-in with Windows Integrated Authentication. If AD FS receives a token request and policy selects Windows Integrated Authentication, AD FS uses this list to determine if it needs to fall back to forms-based authentication. When the user agent for the incoming request is not in this list, AD FS falls back to forms-based authentication.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_extranetlockoutrequirepdc: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether extranet lockout requires a primary domain controller (PDC).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certificatepromotionthreshold: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the period of time, in days, during which a newly generated certificate remains a secondary certificate before being promoted as the primary certificate. The default value is 5 days.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_acceptableidentifiers: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of identifiers that are acceptable names for the Federation Service when it checks the audience for claims that it receives from another claims provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_ntlmonlysupportedclientatproxy: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether to enable support for NTLM-based authentication in situations where the active federation server proxy does not support Negotiate method of authentication. This setting only affects the Windows transport endpoint.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certificaterolloverinterval: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the certificate rollover interval, in minutes. This value determines the frequency at which the Federation Service initiates the rollover service by polling to check whether new certificates need to be generated. The default value is 720 minutes.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_replaycacheexpirationinterval: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the cache duration, in minutes, for token replay detection. This value determines the lifetime for tokens in the replay cache. When the age of a cached token exceeds this interval, the Federation Service determines the token has expired and does not allow replay of it.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_loglevel: {
      type: "Optional[Array[Enum['Errors', 'FailureAudits', 'Information', 'Verbose', 'SuccessAudits', 'Warnings', 'None']]]",
      desc: 'Specifies an array of log detail. The array defines which types of events to record.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_enableoauthdeviceflow: {
      type: 'Optional[Boolean]',
      desc: 'Enabled the OAuth Device Flow.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_persistentssolifetimemins: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the duration, in minutes, of the persistent SSO experience.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_extendedprotectiontokencheck: {
      type: "Optional[Enum['Require', 'Allow', 'None']]",
      desc: "Specifies the level of extended protection for authentication supported by the federation server. Extended Protection for Authentication helps protect against man-in-the-middle (MITM) attacks, in which an attacker intercepts a client's credentials and forwards them to a server. Protection against such attacks is made possible through a Channel Binding Token (CBT) which can be either required, allowed or not required by the server when establishing communications with clients.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_samlmessagedeliverywindow: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the duration, in minutes, for which the Security Assertion Markup Language (SAML) messages sent by the Federation Service are considered valid.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_signedsamlrequestsrequired: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the Federation Service requires signed SAML protocol requests from the relying party. If you specify a value of $True, the Federation Service rejects unsigned SAML protocol requests.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certificatethresholdmultiplier: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the certificate threshold multiplier. By default, this parameter uses the number of minutes in a day (1440) as a multiplier. Change this value only if you want to use a more finely detailed measure of time, such as less than a single day, for calculating the time periods for other certificate threshold parameters.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_auditlevel: {
      type: "Optional[Array[Enum['None', 'Basic', 'Verbose']]]",
      desc: 'Specifies an array of audit levels.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_httpport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the HTTP port for the server.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_enableextranetlockout: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether to enable the lockout algorithm for extranet. When enabled, AD FS checks attributes in Active Directory for the user before validating the credential. If the user is determined to be in lockout state, AD FS will deny the request to the user when accessing from the extranet, to prevent random login attempts from the extranet. Intranet access will continue to be validated against Active Directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_enablerelaystateforidpinitiatedsignon: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that relay state for issuing distribution point (IDP) initiated sign-on is enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_enablelocalauthenticationtypes: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that local authentication types are enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_certificateduration: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the period of time, in days, that any certificates that the Federation Service generates remain valid.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_deviceusagewindowindays: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the length of the device usage window in days.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_intranetuselocalclaimsprovider: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether all web based requests from the intranet default to the default Active Directory claims provider. Use this parameter only when there is more than one claims provider trust in AD FS and you want all user access from the intranet to use the default Active Directory for authentication.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_sendclientrequestidasquerystringparameter: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the client request id, or activity id, is sent as a query string on any redirect from AD FS that is sent to itself. This enables all servers in AD FS to use the same client request id when logging any messages in eventlogs, traces and audits. As a result, it is easier to troubleshoot a single request across multiple AD FS servers in the farm. The default value is $True.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_persistentssocutofftime: {
      type: 'Optional[Timestamp]',
      desc: 'Specifies the earliest issue time of accepted persistent single sign-on (SSO) tokens and OAuth refresh tokens. Persistent SSO tokens or OAuth refresh tokens issued before this time will be rejected. Use this only to reject all prior SSO state across all users and force users to provide fresh credentials.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'DateTime',
      mof_is_embedded: false,
    },
    dsc_federationservicename: {
      type: 'String',
      desc: 'Specifies the DNS name of the federation service.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enableidpinitiatedsignonpage: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether to enable the EnableIdPInitiatedSignonPage property.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_artifactdbconnection: {
      type: 'Optional[String]',
      desc: 'Specifies the connection string to use for the database that maintains the artifacts that the artifact resolution service uses.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_browserssoenabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that browser single sign-on (SSO) is enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_nettcpport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the TCP port number for the server.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_ignoretokenbinding: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether to ignore token binding.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_preventtokenreplays: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the Federation Service prevents the replay of security tokens.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_delegateserviceadministration: {
      type: 'Optional[String]',
      desc: 'Specifies the delegate service administration.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_extranetlockoutthresholdfamiliarlocation: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the maximum number of bad password attempts permitted against the directory before the account is throttled when accessing applications from the extranet for familiar locations. If you use Active Directory Domain Services account lockout policies, it is strongly recommended that you set this threshold to a value that is less than the threshold in AD DS to avoid lockout of the user inside and outside the network. This property is only supported in Windows Server 2019 and above.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_idtokenissuer: {
      type: 'Optional[String]',
      desc: 'Specifies the URI of the token issuer.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowsystemserviceadministration: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that system service administration is allowed.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_enablepersistentsso: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether to store the SSO token in persistent cookies for devices joined to a workplace.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_allowlocaladminsserviceadministration: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that local administrator service administration is allowed.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_authenticationcontextorder: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of authentication contexts, in order of relative strength. Specify each authentication context as a URI.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_hostname: {
      type: 'Optional[String]',
      desc: 'Specifies the network addressable host name of the Federation Service.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_extranetlockoutthreshold: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the maximum number of bad password attempts permitted against the directory before the account is throttled when accessing applications from the extranet for unfamiliar locations. If you use Active Directory Domain Services account lockout policies, it is strongly recommended that you set this threshold to a value that is less than the threshold in AD DS to avoid lockout of the user inside and outside the network.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_autocertificaterollover: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the system manages certificates for the administrator and generates new certificates before the expiration date of current certificates.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_browserssosupporteduseragents: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of user agents that are supported for browser SSO.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_kmsilifetimemins: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the lifetime of the sign on status for KMSI.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_httpsport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the HTTPS port for the server.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_additionalerrorpageinfo: {
      type: "Optional[Enum['Private', 'Detailed', 'None']]",
      desc: 'Specifies the level of additional information that is displayed on ADFS error pages.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_identifier: {
      type: 'Optional[String]',
      desc: 'Specifies the URI that uniquely identifies the Federation Service.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_loopdetectiontimeintervalinseconds: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the time interval in seconds for AD FS to track multiple token requests that are occurring and being rejected by the relying party causing a redirect back to AD FS for a new token request. Use in conjunction with the LoopDetectionMaximumTokensIssuedInInterval parameter.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_ssolifetime: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the duration, in minutes, of the single sign-on (SSO) experience for Web browser clients.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_loopdetectionmaximumtokensissuedininterval: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the maximum number of tokens that can be issued within the time period specified by the LoopDetectionTimeIntervalInSeconds parameter before AD FS will reject the request and present an error to the user. Use in conjunction with the LoopDetectionMaximumTokensIssuedInInterval parameter.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
  },
)
