require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfsrelyingpartytrust',
  dscmeta_resource_friendly_name: 'AdfsRelyingPartyTrust',
  dscmeta_resource_name: 'MSFT_AdfsRelyingPartyTrust',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsRelyingPartyTrust resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_encryptclaims: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the claims that are sent to the relying party are encrypted.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_additionalwsfedendpoint: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of alternate return addresses for the application. This is typically used when the application wants to indicate to AD FS what the return URL should be on successful token generation. AD FS requires that all acceptable URLs are entered as trusted information by the administrator.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_delegationauthorizationrules: {
      type: 'Optional[String]',
      desc: 'Specifies the delegation authorization rules for issuing claims to this relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_issuanceauthorizationrules: {
      type: 'Optional[String]',
      desc: 'Specifies the issuance authorization rules for issuing claims to this relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_encryptioncertificaterevocationcheck: {
      type: "Optional[Enum['None', 'CheckEndCert', 'CheckEndCertCacheOnly', 'CheckChain', 'CheckChainCacheOnly', 'CheckChainExcludeRoot', 'CheckChainExcludeRootCacheOnly']]",
      desc: 'Specifies the type of validation that should occur for the encryption certificate it is used for encrypting claims to the relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_impersonationauthorizationrules: {
      type: 'Optional[String]',
      desc: 'Specifies the impersonation authorization rules for issuing claims to this relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the relying party trust is enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_encryptednameidrequired: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the relying party requires that the NameID claim be encrypted.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_allowedauthenticationclassreferences: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of allow authentication class references.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_additionalauthenticationrules: {
      type: 'Optional[String]',
      desc: 'Specifies the additional authorization rules to require additional authentication based on user, device and location attributes after the completion of the first step of authentication. Note: These rules must only be configured after there is at least one authentication provider enabled for additional authentication.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_wsfedendpoint: {
      type: 'Optional[String]',
      desc: 'Specifies the WS-Federation Passive URL for this relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_samlresponsesignature: {
      type: "Optional[Enum['AssertionOnly', 'MessageAndAssertion', 'MessageOnly']]",
      desc: 'Specifies the response signature or signatures that the relying party expects.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_accesscontrolpolicyname: {
      type: 'Optional[String]',
      desc: 'Specifies the name of an access control policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_issuancetransformrules: {
      type: "Optional[Array[Struct[{
              customrule => Optional[String],
              templatename => Enum['LdapClaims', 'EmitGroupClaims', 'CustomClaims'],
              attributestore => Optional[String],
              outgoingnameidformat => Optional[String],
              name => String,
              groupname => Optional[String],
              outgoingclaimvalue => Optional[String],
              ldapmapping => Array[Struct[{
                outgoingclaimtype => String,
                ldapattribute => String,
                cim_instance_type => Enum['MSFT_AdfsLdapMapping']
              }]],
              outgoingclaimtype => Optional[String]
            }]]]",
      desc: 'Specifies the issuance transform rules for issuing claims to this relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_AdfsIssuanceTransformRule[]',
      mof_is_embedded: true,
    },
    dsc_requestmfafromclaimsproviders: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether to use the request MFA from claims providers option.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_accesscontrolpolicyparameters: {
      type: 'Optional[Struct[{
              groupparameter => Optional[Array[String]],
            }]]',
      desc: 'Specifies the parameters and their values to pass to the Access Control Policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_AdfsAccessControlPolicyParameters',
      mof_is_embedded: true,
    },
    dsc_identifier: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies the unique identifiers for this relying party trust. No other trust can use an identifier from this list. Uniform Resource Identifiers (URIs) are often used as unique identifiers for a relying party trust, but you can use any string of characters.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_refreshtokenprotectionenabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether refresh token protection is enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_monitoringenabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether periodic monitoring of this relying party federation metadata is enabled. The MetadataUrl parameter specifies the URL of the relying party federation metadata.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Specifies whether to remove or add the relying party trust.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_claimaccepted: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of claims that this relying party accepts.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_tokenlifetime: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the duration, in minutes, for which the claims that are issued to the relying party are valid.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_protocolprofile: {
      type: "Optional[Enum['SAML', 'WsFederation', 'WsFed-SAML']]",
      desc: 'Specifies which protocol profiles the relying party supports.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_signedsamlrequestsrequired: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the Federation Service requires signed SAML protocol requests from the relying party. If you specify a value of True, the Federation Service rejects unsigned SAML protocol requests.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_issueoauthrefreshtokensto: {
      type: "Optional[Enum['NoDevice', 'WorkplaceJoinedDevices', 'AllDevices']]",
      desc: 'Specifies the refresh token issuance device types.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_samlendpoint: {
      type: "Optional[Array[Struct[{
              binding => Optional[Enum['Artifact', 'POST', 'Redirect', 'SOAP']],
              uri => Optional[String],
              index => Optional[Integer[-2147483648, 2147483647]],
              responseuri => Optional[String],
              isdefault => Optional[Boolean],
              protocol => Optional[Enum['SAMLArtifactResolution', 'SAMLAssertionConsumer', 'SAMLLogout', 'SAMLSingleSignOn']]
            }]]]",
      desc: 'Specifies an array of Security Assertion Markup Language (SAML) protocol endpoints for this relying party.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_AdfsSamlEndpoint[]',
      mof_is_embedded: true,
    },
    dsc_allowedclienttypes: {
      type: "Optional[Array[Enum['None', 'Public', 'Confidential']]]",
      desc: 'Specifies allowed client types.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_enablejwt: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the JSON Web Token (JWT) format should be used to issue a token on a WS-Federation request. By default, SAML tokens are issued over WS-Federation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_signaturealgorithm: {
      type: "Optional[Enum['http://www.w3.org/2000/09/xmldsig#rsa-sha1', 'http://www.w3.org/2001/04/xmldsig-more#rsa-sha256']]",
      desc: 'Specifies the signature algorithm that the relying party uses for signing and verification.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_claimsprovidername: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies the name of the claim provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_metadataurl: {
      type: 'Optional[String]',
      desc: 'Specifies a URL at which the federation metadata for this relying party trust is available.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_autoupdateenabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether changes to the federation metadata by the MetadataURL parameter apply automatically to the configuration of the trust relationship. If this parameter has a value of True, partner claims, certificates, and endpoints are updated automatically.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_alwaysrequireauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Indicates to always require authentication.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_notbeforeskew: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the skew, as in integer, for the time stamp that marks the beginning of the validity period.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_notes: {
      type: 'Optional[String]',
      desc: 'Specifies notes for this relying party trust.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Specifies the friendly name of this relying party trust.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_signingcertificaterevocationcheck: {
      type: "Optional[Enum['None', 'CheckEndCert', 'CheckEndCertCacheOnly', 'CheckChain', 'CheckChainCacheOnly', 'CheckChainExcludeRoot', 'CheckChainExcludeRootCacheOnly']]",
      desc: 'Specifies the type of certificate validation that occur when signatures on requests from the relying party are verified.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
