require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adfswebapiapplication',
  dscmeta_resource_friendly_name: 'AdfsWebApiApplication',
  dscmeta_resource_name: 'MSFT_AdfsWebApiApplication',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'AdfsDsc',
  dscmeta_module_version: '1.1.0',
  docs: 'The DSC AdfsWebApiApplication resource type.
         Automatically generated from version 1.1.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_applicationgroupidentifier: {
      type: 'String',
      desc: 'Specifies the ID of an application group for the Web API application.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_claimsprovidername: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of claims provider names that you can configure for a relying party trust for Home Realm Discovery (HRD) scenario.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_issueoauthrefreshtokensto: {
      type: "Optional[Enum['NoDevice', 'WorkplaceJoinedDevices', 'AllDevices']]",
      desc: 'Specifies the refresh token issuance device types.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Specifies a name for the Web API application.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_accesscontrolpolicyname: {
      type: 'Optional[String]',
      desc: 'Specifies the name of an access control policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_refreshtokenprotectionenabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether refresh token protection is enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_impersonationauthorizationrules: {
      type: 'Optional[String]',
      desc: 'Specifies the impersonation authorization rules.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_issuanceauthorizationrules: {
      type: 'Optional[String]',
      desc: 'Specifies the issuance authorization rules.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: 'Specifies a description for the Web API application.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_additionalauthenticationrules: {
      type: 'Optional[String]',
      desc: 'Specifies additional authentication rules.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: "Specifies whether the Web API application should be present or absent. Default value is 'Present'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_identifier: {
      type: 'Array[String]',
      desc: 'Specifies an identifier for the Web API application.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_allowedclienttypes: {
      type: "Optional[Array[Enum['None', 'Public', 'Confidential']]]",
      desc: 'Specifies allowed client types.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_issuancetransformrules: {
      type: "Optional[Array[Struct[{
              customrule => Optional[String],
              templatename => Enum['LdapClaims', 'EmitGroupClaims', 'CustomClaims'],
              attributestore => Optional[String],
              outgoingnameidformat => Optional[String],
              name => String,
              groupname => Optional[String],
              outgoingclaimvalue => Optional[String],
              ldapmapping => Array[Struct[{
                outgoingclaimtype => String,
                ldapattribute => String,
                cim_instance_type => Enum['MSFT_AdfsLdapMapping']
              }]],
              outgoingclaimtype => Optional[String]
            }]]]",
      desc: 'Specifies the issuance transform rules.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_AdfsIssuanceTransformRule[]',
      mof_is_embedded: true,
    },
    dsc_accesscontrolpolicyparameters: {
      type: 'Optional[Struct[{
              groupparameter => Optional[Array[String]],
            }]]',
      desc: 'Specifies the parameters and their values to pass to the Access Control Policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_AdfsAccessControlPolicyParameters',
      mof_is_embedded: true,
    },
    dsc_allowedauthenticationclassreferences: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies an array of allow authentication class references.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_requestmfafromclaimsproviders: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that the request MFA from claims providers option is used.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_tokenlifetime: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the token lifetime.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_delegationauthorizationrules: {
      type: 'Optional[String]',
      desc: 'Specifies delegation authorization rules.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_alwaysrequireauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that this Web API application role always requires authentication, even if it previously authenticated credentials for access. Specify this parameter to require users to always supply credentials to access sensitive resources.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_notbeforeskew: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the not before skew value.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
  },
)
